def display_via_matplotlib(images, title):
    """Display an animated sequence of images using Matplotlib."""
    import matplotlib.pyplot as plt
    import matplotlib.animation as animation

    fig = plt.figure(figsize=(images[0].width/100,images[1].height/100))
    ims = []
    for i in range(len(images)):
        im = plt.imshow(images[i], animated=True)
        ims.append([im])
    ani = animation.ArtistAnimation(fig, ims, interval=50, blit=True,
                                    repeat_delay=0)
    plt.axis('off')    
    plt.axis("tight")
    plt.axis("image")
    fig.tight_layout(pad=0)
    if title is not None:
        fig.canvas.set_window_title(title)
    plt.show()
    return ani



def display_via_videofig(images, title):
    """Display an animated sequence of images in a new window."""
    from videofig import videofig
    # redraw_fn draw frame f in a image sequence
    def redraw_fn(i, axes):
        if not redraw_fn.initialized:
            redraw_fn.im = axes.imshow(images[i], animated=True)
            redraw_fn.initialized = True
        else:
            redraw_fn.im.set_array(images[i])
    redraw_fn.initialized = False
    return videofig(len(images), redraw_fn, play_fps=30)



try:
    from .display_via_tk import display_tk_multiprocess
    from .display_via_matplotlib import display_via_matplotlib
    from .display_via_moviepy import display_via_moviepy
    from .display_using_moviepy_and_browser_launch import display_using_moviepy_and_browser_launch
    from .display_via_videofig import display_via_videofig
except ImportError:
    from display_via_tk import display_tk_multiprocess
    from display_via_matplotlib import display_via_matplotlib
    from display_via_moviepy import display_via_moviepy
    from display_using_moviepy_and_browser_launch import display_using_moviepy_and_browser_launch
    from display_via_videofig import display_via_videofig

options =  ['tk_multiprocess',
            'matplotlib',
            'moviepy',
            'moviepy_launch_browser',
            'videofig']

def display(images, title, via='matplotlib'):
    """Display an animated sequence of images via a chosen a method.
    Most methods allow the continued use of the Python terminal.
    `via` can be one of ['tk_multiprocess',
                         'matplotlib',
                         'moviepy',
                         'moviepy_launch_browser'],
                         'videofig'"""
    if via == 'tk_multiprocess':
        display_tk_multiprocess(images, title)
    elif via == 'matplotlib':
        return display_via_matplotlib(images, title)
    elif via == 'moviepy':
        display_via_moviepy(images, title)
    elif via == 'moviepy_launch_browser':
        display_using_moviepy_and_browser_launch(images, title)
    elif via == 'videofig':
        display_via_videofig(images, title)
    else:
        raise ValueError("""`via` must be one of """ + str(options))


import tkinter as tk
from itertools import count
from PIL import ImageTk
from tkinter.filedialog import asksaveasfilename
import tkinter.font as font
from multiprocessing import Process
    
def display_tk_multiprocess(images, title):
    Process(target=do_display_tk_multiprocess, args=(images,title)).start()

def do_display_tk_multiprocess(images, title):
    root = tk.Tk()
    if title is not None:
        root.title(title)
    menubar = tk.Menu(root)
    filemenu = tk.Menu(menubar, tearoff=0)
    filemenu.add_command(label="Save As...",
                         command=lambda: save_animation(images))
    menubar.add_cascade(label="File", menu=filemenu)
    lbl = ImageLabel(root)
    lbl.pack()
    lbl.load(images)
    lbl.configure(background='black')
    #root.configure(background='gray')
    pause_play = tk.Button(root,
                           text="⏯️",
                           font=font.Font(size=30),
                           command=lambda: lbl.pause_play())
    pause_play.pack(expand=True, fill=tk.X, side=tk.RIGHT)
    root.config(menu=menubar)
    root.mainloop()

class ImageLabel(tk.Label):
    """A label that displays images, and plays them if there are more
    than one."""
    def load(self, im):
        self.loc = 0
        self.frames = []
        self.paused = False
        try:
            for i in count(0):
                self.frames.append(ImageTk.PhotoImage(im[i]))
        except IndexError:
            pass

        try:
            self.delay = im.info['duration']
        except:
            self.delay = 50
        if len(self.frames) == 1:
            self.config(image=self.frames[0])
        else:
            self.next_frame()
            
    def pause_play(self):
        self.paused = not self.paused
        self.next_frame()
        
    def unload(self):
        self.config(image=None)
        self.frames = None

    def next_frame(self):
        if self.frames and not self.paused:
            self.loc += 1
            self.loc %= len(self.frames)
            self.config(image=self.frames[self.loc])
            self.after(self.delay, self.next_frame)

def save_animation(images):
    """Saves an animation (usually as a GIF)."""
    ftypes = [('GIF file', '.gif'),
              ('All files', '*')]
    filename = asksaveasfilename(filetypes=ftypes, defaultextension='.gif')
    if filename != () and filename != '':
        images[0].save(filename, save_all=True, loop=0,
              append_images = images[1:])
        



def display_using_moviepy_and_browser_launch(images, title):
    """Display an animated sequence of images in the browser using moviepy."""
    from moviepy.editor import ImageSequenceClip
    import numpy as np
    import webbrowser
    import tempfile
    from time import sleep
    as_arrays = [np.array(image) for image in images]
    the_html = ImageSequenceClip(as_arrays, fps=24).ipython_display()
    with tempfile.NamedTemporaryFile(mode='w+', suffix='.html') as f:
        f.write(the_html.data)
        webbrowser.open(f.name)
        # This is to allow the browser time to open the file before it gets destroyed
        # It's a hack
        sleep(1)


from distutils.core import setup
setup(
  name = 'laguerre_transformations',
  packages = ['laguerre_transformations'],
  version = '0.3',
  license='BSD-3-Clause',
  description = 'Visualize Laguerre transformations.',
  author = 'Ran Gutin',
  author_email = 'jkabrg@gmail.com',
  url = 'https://github.com/ogogmad/laguerre_transformations',
  download_url = 'https://github.com/ogogmad/laguerre_transformations/archive/v_03.tar.gz',
  keywords = ['mathematics', 'transformations', 'geometry', 'hypercomplex'],
  install_requires=[
          'scipy',
          'Pillow',
          'matplotlib'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Science/Research',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: BSD License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)



def display_via_moviepy(images, title):
    """Display an animated sequence of images in the browser using moviepy."""
    from moviepy.editor import ImageSequenceClip
    import numpy as np

    as_arrays = [np.array(image) for image in images]
    return ImageSequenceClip(as_arrays, fps=24).ipython_display()
